"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var PactNative = require("../native");
var PactV3 = /** @class */ (function () {
    function PactV3(opts) {
        this.states = [];
        this.opts = opts;
        this.pact = new PactNative.Pact(opts.consumer, opts.provider);
    }
    PactV3.prototype.given = function (providerState, parameters) {
        this.states.push({ description: providerState, parameters: parameters });
        return this;
    };
    PactV3.prototype.uponReceiving = function (desc) {
        this.pact.addInteraction(desc, this.states);
        return this;
    };
    PactV3.prototype.withRequest = function (req) {
        this.pact.addRequest(req, req.body && JSON.stringify(req.body));
        return this;
    };
    PactV3.prototype.withRequestBinaryFile = function (req, contentType, file) {
        this.pact.addRequestBinaryFile(req, contentType, file);
        return this;
    };
    PactV3.prototype.withRequestMultipartFileUpload = function (req, contentType, file, part) {
        this.pact.addRequestMultipartFileUpload(req, contentType, file, part);
        return this;
    };
    PactV3.prototype.willRespondWith = function (res) {
        this.pact.addResponse(res, res.body && JSON.stringify(res.body));
        this.states = [];
        return this;
    };
    PactV3.prototype.withResponseBinaryFile = function (res, contentType, file) {
        this.pact.addResponseBinaryFile(res, contentType, file);
        return this;
    };
    PactV3.prototype.withResponseMultipartFileUpload = function (req, contentType, file, part) {
        this.pact.addResponseMultipartFileUpload(req, contentType, file, part);
        return this;
    };
    PactV3.prototype.executeTest = function (testFn) {
        var _this = this;
        var result = this.pact.executeTest(testFn);
        if (result.testResult) {
            return result.testResult
                .then(function (val) {
                var testResult = _this.pact.getTestResult(result.mockServer.id);
                if (testResult.mockServerError) {
                    return Promise.reject(new Error(testResult.mockServerError));
                }
                else if (testResult.mockServerMismatches) {
                    var error = "Mock server failed with the following mismatches: ";
                    for (var _i = 0, _a = testResult.mockServerMismatches; _i < _a.length; _i++) {
                        var mismatch = _a[_i];
                        error += "\n\t" + mismatch;
                    }
                    return Promise.reject(new Error(error));
                }
                else {
                    _this.pact.writePactFile(result.mockServer.id, _this.opts.dir);
                    return val;
                }
            })
                .catch(function (err) {
                var testResult = _this.pact.getTestResult(result.mockServer.id);
                var error = "Test failed for the following reasons:";
                error += "\n\n\tTest code failed with an error: " + err.message;
                if (testResult.mockServerError) {
                    error += "\n\n\t" + testResult.mockServerError;
                }
                if (testResult.mockServerMismatches) {
                    error += "\n\n\tMock server failed with the following mismatches: ";
                    var i = 1;
                    for (var _i = 0, _a = testResult.mockServerMismatches; _i < _a.length; _i++) {
                        var mismatchJson = _a[_i];
                        var mismatches = JSON.parse(mismatchJson);
                        if (mismatches.mismatches) {
                            for (var _b = 0, _c = mismatches.mismatches; _b < _c.length; _b++) {
                                var mismatch = _c[_b];
                                error += "\n\t\t" + i++ + ") " + mismatch.type + " " + (mismatch.path ? "(at " + mismatch.path + ") " : "") + mismatch.mismatch;
                            }
                        }
                    }
                }
                return Promise.reject(new Error(error));
            })
                .finally(function () {
                _this.pact.shutdownTest(result);
            });
        }
        else {
            this.pact.shutdownTest(result);
            return Promise.reject(result.testError);
        }
    };
    return PactV3;
}());
exports.PactV3 = PactV3;
//# sourceMappingURL=pact.js.map